/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2013-2017 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::functionObjects::setTimeStepViscAutoFunctionObject

Group
    grpUtilitiesFunctionObjects

Description
    Overrides the timeStep. Can only be used with
    solvers with adjustTimeStep control (e.g. pimpleFoam). Makes no attempt
    to cooperate with other timeStep 'controllers' (maxCo, other
    functionObjects). Supports 'enabled' flag but none of the other ones
    'timeStart', 'timeEnd', 'writeControl' etc.

SourceFiles
    setTimeStepViscAutoFunctionObject.C

\*---------------------------------------------------------------------------*/

#ifndef functionObjects_setTimeStepViscAutoFunctionObject_H
#define functionObjects_setTimeStepViscAutoFunctionObject_H

//#include "functionObject.H"
#include "fvMeshFunctionObject.H"
#include "volFields.H"
#include "Function1.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
namespace functionObjects
{

/*---------------------------------------------------------------------------*\
                    Class setTimeStepViscAutoFunctionObject Declaration
\*---------------------------------------------------------------------------*/

class setTimeStepViscAutoFunctionObject
:
    public fvMeshFunctionObject
//    public functionObject
{
    // Private data

        //- Reference to the time database
        const Time& time_;

        //- Time step function (or table: this is why it is implemented like this) 
        autoPtr<Function1<scalar>> timeStepPtr_;
        autoPtr<Function1<scalar>> viscDimLessTimePtr_;

        scalar minViscTime_;
        scalar currentTimeStep_;
        scalar relaxationTimeFactor_;
        bool   hasBeenExecuted_;

    // Private member functions

        //- Disallow default bitwise copy construct
        setTimeStepViscAutoFunctionObject(const setTimeStepViscAutoFunctionObject&);

        //- Disallow default bitwise assignment
        void operator=(const setTimeStepViscAutoFunctionObject&);


public:

    //- Runtime type information
    TypeName("setTimeStepViscAuto");


    // Constructors

        //- Construct from components
        setTimeStepViscAutoFunctionObject
        (
            const word& name,
            const Time& runTime,
            const dictionary& dict
        );


    // Destructor
    virtual ~setTimeStepViscAutoFunctionObject();


    // Member Functions

        //- Return time database
        const Time& time() const;

        //- Called at the end of Time::adjustDeltaT() if adjustTime is true
        virtual bool adjustTimeStep();

        //- Read and set the function object if its data have changed
        virtual bool read(const dictionary&);

        //- Called at each ++ or += of the time-loop.
        //  postProcess overrides the usual executeControl behaviour and
        //  forces execution (used in post-processing mode)
        virtual bool execute();

        //- Called at each ++ or += of the time-loop.
        //  postProcess overrides the usual writeControl behaviour and
        //  forces writing always (used in post-processing mode)
        virtual bool write();
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace functionObjects
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
